/* -*- mode: C; indent-tabs-mode: t; tab-width: 2; c-basic-offset: 2; -*- */

/* same-gnome.c : Same Game
 *
 * Copyright (c) 2004 by Callum McKenzie
 *
 */

#include "config.h"

#include <gnome.h>

#include <games-gconf.h>
#include <games-gridframe.h>
#include <games-stock.h>
#include <games-scores.h>

#include "same-gnome.h"

#include "drawing.h"
#include "game.h"
#include "input.h"
#include "ui.h"

#define DEFAULT_WINDOW_WIDTH 450
#define DEFAULT_WINDOW_HEIGHT 350

#define DEFAULT_CUSTOM_WIDTH 15
#define DEFAULT_CUSTOM_HEIGHT 10
#define MINIMUM_CUSTOM_WIDTH 4
#define MINIMUM_CUSTOM_HEIGHT 4
#define MAXIMUM_CUSTOM_HEIGHT 100
#define MAXIMUM_CUSTOM_WIDTH 100

gchar *localthemedir;

gchar *theme;
gint   game_size = UNSET;

GConfClient *gcclient;

/* Keep this in sync with the enum above. */
gint board_sizes[MAX_SIZE][3] = {{-1, -1, -1}, /* This is a dummy entry. */
																 {-1, -1, -1}, /* Space for the custom size. */
																 {15, 10, 3},
																 {30, 20, 4},
																 {45, 30, 4}};

const GamesScoresCategory scorecats[] = {{"Small", N_("Small")},
																				 {"Medium", N_("same-gnome|Medium")},
																				 {"Large", N_("Large")},
																				 GAMES_SCORES_LAST_CATEGORY};

static const GamesScoresDescription scoredesc = {scorecats,
																								 "Small",
																								 "same-gnome",
																								 GAMES_SCORES_STYLE_PLAIN_DESCENDING};

GamesScores *highscores;
gint board_width;
gint board_height;
gint board_ncells;
gint ncolours;

gint window_width;
gint window_height;

static void initialise_options (gint requested_size, gchar *requested_theme)
{
  gint intvalue;

  game_size = SMALL;
  theme = DEFAULT_THEME;

  if (requested_size != -1)
    game_size = requested_size;
  if (requested_theme != NULL)
    theme = requested_theme;

	localthemedir = g_build_filename (g_get_user_data_dir (), 
																		"/gnome-games/same-gnome/themes/",
																		THEME_VERSION, NULL);

  gcclient = gconf_client_get_default ();
  games_gconf_sanity_check_string (gcclient, GCONF_THEME_KEY);

  intvalue = gconf_client_get_int (gcclient, GCONF_CUSTOM_WIDTH_KEY, NULL);
  if (intvalue == 0)
    intvalue = DEFAULT_CUSTOM_WIDTH;
  board_sizes[CUSTOM][0] = CLAMP (intvalue, MINIMUM_CUSTOM_WIDTH, 
				  MAXIMUM_CUSTOM_WIDTH);
  intvalue = gconf_client_get_int (gcclient, GCONF_CUSTOM_HEIGHT_KEY, NULL);
  if (intvalue == 0)
    intvalue = DEFAULT_CUSTOM_HEIGHT;
  board_sizes[CUSTOM][1] = CLAMP (intvalue, MINIMUM_CUSTOM_HEIGHT, 
				  MAXIMUM_CUSTOM_HEIGHT);

  if (requested_size != -1) 
    game_size = requested_size;
  else {
    game_size = gconf_client_get_int (gcclient, GCONF_SIZE_KEY, NULL);
    if (game_size == 0)
      game_size = DEFAULT_GAME_SIZE;
  }

	/* FIXME: This doesn't work for a custom size. */
  game_size = CLAMP (game_size, SMALL, MAX_SIZE - 1);
	set_sizes (game_size);

  intvalue = gconf_client_get_int (gcclient, GCONF_WINDOW_WIDTH_KEY, NULL);
  if (intvalue == 0)
    intvalue = DEFAULT_WINDOW_WIDTH;
  window_width = intvalue;

  intvalue = gconf_client_get_int (gcclient, GCONF_WINDOW_HEIGHT_KEY, NULL);
  if (intvalue == 0)
    intvalue = DEFAULT_WINDOW_HEIGHT;
  window_height = intvalue;

  if (requested_theme != NULL)
    theme = requested_theme;
  else
    theme = games_gconf_get_string (gcclient, GCONF_THEME_KEY, DEFAULT_THEME);

  /* An invalid theme will be picked up at load time. Although we can
   * guarantee that theme != NULL. */
}

int main (int argc, char *argv[])
{
  static gchar *requested_theme = NULL;
  static gint   requested_size  = -1;

  static const struct poptOption options[] = {
    { "theme", 't', POPT_ARG_STRING, &requested_theme, 0, N_("Set the theme"),
      N_("NAME") },
    { "scenario", 's', POPT_ARG_STRING, &requested_theme, 0, 
      N_("For backwards compatibility"), N_("NAME") },
    { "size", 'z', POPT_ARG_INT, &requested_size, 0, 
      N_("Game size (1=small, 3=large)"), N_("NUMBER") },
    { NULL, '\0', 0, NULL, 0, NULL, NULL }};

	setgid_io_init ();

  /* Initialise i18n. */
  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
  textdomain (GETTEXT_PACKAGE);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");

  /* Initialise GNOME. */
  gnome_program_init (APPNAME, VERSION, LIBGNOMEUI_MODULE, argc, argv,
											GNOME_PARAM_APP_DATADIR, DATADIR,
											GNOME_PARAM_POPT_TABLE, options,  
											NULL);

	games_stock_init ();

	highscores = games_scores_new (&scoredesc);

  initialise_options (requested_size, requested_theme);

  build_gui ();

	/* FIXME: This should be one alternative of an if statement, the other
	 * alternative should be to load an old game. */
	if (!load_game ())
		new_game ();
	

  gtk_main ();

  gnome_accelerators_sync();

  return 0;
}
